#include <os.h>
#include "output.h"

#define REAL_TIME_CLOCK_ADDRESS ((uint32_t*)0x90090000)
#define SCREEN_FRAME_BASE_ADDRESS ((void**)0xC0000010)

int printtime(uint8_t* scrbuf, int x, int y, unsigned millisec){
	unsigned ds = (millisec/100) % 10;
	unsigned s = (millisec/1000) % 60;
	unsigned m = (millisec/60000) % 60;
	unsigned h = (millisec/3600000);
	if(h > 99)
		return 1;
	char time[] = "00:00:00.0";
	if(h == 0)
		sprintf(time, "%02i:%02i.%i", m, s, ds);
	else
		sprintf(time, "%02i:%02i:%02i.%i", h, m, s, ds);
	dispString(scrbuf, x, y, time);
	return 0;
}

int main(){
	clrscr();
	lcd_ingray();
    uint8_t* uascrbuf = (uint8_t*) malloc(SCREEN_BYTES_SIZE+8);
	uint8_t* scrbuf = uascrbuf - ((uint32_t)uascrbuf % 8) + 8;
	uint8_t* restoreframeaddress = *SCREEN_FRAME_BASE_ADDRESS;
	*SCREEN_FRAME_BASE_ADDRESS = scrbuf;
    memset(scrbuf, 0xFF, SCREEN_BYTES_SIZE);
    unsigned sec = 0;
	unsigned ms = 0;
	unsigned oldtime = 0;
	unsigned offset = 0;//how many milliseconds into an rtc second did we start?
	int running = 0;
	int key_already_pressed = 0;

	dispString(scrbuf, 20, 180, "[ENTER] Start");
	dispString(scrbuf, 20, 200, " [MENU] Reset");
    dispString(scrbuf, 20, 220, "  [ESC] Close program ");

    while(!isKeyPressed(KEY_NSPIRE_ESC)){
		if(isKeyPressed(KEY_NSPIRE_ENTER) && !key_already_pressed){//start or stop
			if(!running){//on start
				dispString(scrbuf, 20, 180, "[ENTER] Stop ");
				oldtime = *REAL_TIME_CLOCK_ADDRESS;
			}
			else{//on stop
				if(ms >= offset)
					ms -= offset;
				else if(sec > 0){
					sec--;
					ms = ms + 1000 - offset;
				}
				else{
					ms = 0;
					sec = 0;
				}
				offset = 0;
				dispString(scrbuf, 20, 180, "[ENTER] Start");
			}
			running = !running;
			key_already_pressed = 1;
		}
		if(isKeyPressed(KEY_NSPIRE_MENU) && !key_already_pressed){//reset
			sec = 0;
			ms = 0;
			offset = 0;
			oldtime = *REAL_TIME_CLOCK_ADDRESS;
			key_already_pressed = 1;
		}
		if(!any_key_pressed())
			key_already_pressed = 0;

		if(running){
			if(*REAL_TIME_CLOCK_ADDRESS != oldtime){
				sec++;
				oldtime = *REAL_TIME_CLOCK_ADDRESS;
				if(!offset)
					offset = 1000 - ms;
				ms = 0;
			}
			ms+=10;
		}

    	printtime(scrbuf, 130, 60, sec*1000 + ms - offset);
		sleep(10);
    }

	*SCREEN_FRAME_BASE_ADDRESS = restoreframeaddress;
	free(uascrbuf);
	return 0;
}
